import { isDrawBarData, getCanvasPoint } from './core';
export function subscriber(params) {
    const { canvas } = params;
    return (event, func, callback) => {
        const handler = callback && func && func(params);
        if (handler)
            canvas.addEventListener(event, handler);
        return () => handler && canvas.removeEventListener(event, handler);
    };
}
function findDataIndex(ctx, paths, cX, cY) {
    for (let i = 0; i < paths.length; i++) {
        if (ctx.isPointInPath(paths[i], cX, cY))
            return i;
    }
    return undefined;
}
function clickHandler(params) {
    const canvas = params.canvas;
    const { ratio, onClick } = params.options;
    return (event) => {
        if (!onClick)
            return;
        let dataIndex;
        const ctx = canvas.getContext('2d');
        const [cX, cY] = getCanvasPoint(canvas, ratio, event);
        for (let i = 0; i < params.drawData.length; i++) {
            const item = params.drawData[i];
            if (ctx.isPointInPath(item.mask, cX, cY)) {
                if (isDrawBarData(item))
                    dataIndex = findDataIndex(ctx, item.bars, cX, cY);
                onClick(item.data, dataIndex);
                return;
            }
        }
    };
}
function moveHandler(drawFunc, getData) {
    return (params) => {
        const canvas = params.canvas;
        const ctx = canvas.getContext('2d');
        const { ratio, onClick, onHoverChange } = params.options;
        return (event) => {
            if (!onHoverChange)
                return;
            let cursor = 'default';
            let found;
            let dataIndex;
            const [cX, cY] = getCanvasPoint(canvas, ratio, event);
            drawFunc(params);
            const { items, fill } = getData();
            for (let i = 0; i < items.length; i++) {
                const item = items[i];
                if (ctx.isPointInPath(item.mask, cX, cY)) {
                    ctx.fillStyle = fill(item.data.color);
                    ctx.fill(item.mask);
                    cursor = 'pointer';
                    found = item.data;
                    if (isDrawBarData(item)) {
                        dataIndex = findDataIndex(ctx, item.bars, cX, cY);
                    }
                    break;
                }
            }
            onHoverChange(found && Object.assign({ data: found, clientX: event.clientX, clientY: event.clientY }, (dataIndex !== undefined && { dataIndex })));
            if (onClick)
                canvas.style.cursor = cursor;
        };
    };
}
function leaveHandler(drawFunc) {
    return (params) => {
        const canvas = params.canvas;
        const { onClick, onHoverChange } = params.options;
        return () => {
            drawFunc(params);
            if (onHoverChange)
                onHoverChange(undefined);
            if (onClick)
                canvas.style.cursor = 'default';
        };
    };
}
export function setupEvents(params, drawFunc, getData) {
    const sub = subscriber(params);
    const { onClick, onHoverChange } = params.options;
    const unsubClick = sub('click', clickHandler, onClick);
    const unsubMove = sub('mousemove', moveHandler(drawFunc, getData), onHoverChange);
    const unsubLeave = sub('mouseleave', leaveHandler(drawFunc), onHoverChange);
    return () => {
        unsubClick();
        unsubMove();
        unsubLeave();
    };
}
